from durakgame import CardValue, Suit

six = CardValue.Six
seven = CardValue.Seven
eight = CardValue.Eight
nine = CardValue.Nine
ten = CardValue.Ten
jack = CardValue.Jack
queen = CardValue.Queen
king = CardValue.King
ace = CardValue.Ace

heart = Suit.Heart
spade = Suit.Spade
cross = Suit.Cross
diamond = Suit.Diamond
