import random
import unittest
from durakgame import *

from .utils import *

class MoveGenerationTest(unittest.TestCase):
    def setUp(self):
        pass

    def test1(self):
        pass

class MoveProcessingTest(unittest.TestCase):
    def setUp(self):
        pass
    
    def test1(self):
        pass

if __name__ == "__main__":
    unittest.main()
